package sun.security.krb5.internal.ccache;

import sun.security.krb5.internal.Krb5;
import sun.security.krb5.Asn1Exception;
import sun.security.krb5.internal.Ticket;
import sun.security.krb5.internal.AuthorizationData;
import sun.security.krb5.internal.HostAddresses;
import sun.security.krb5.internal.TicketFlags;
import sun.security.krb5.internal.KerberosTime;
import sun.security.krb5.internal.AuthorizationDataEntry;
import sun.security.krb5.internal.KrbApErrException;
import sun.security.krb5.internal.HostAddress;
import sun.security.krb5.EncryptionKey;
import java.util.StringTokenizer;
import sun.security.krb5.RealmException;
import sun.security.krb5.Realm;
import sun.misc.IOUtils;
import java.util.ArrayList;
import sun.security.krb5.PrincipalName;
import java.io.IOException;
import java.io.InputStream;
import sun.security.krb5.internal.util.KrbDataInputStream;

public class CCacheInputStream extends KrbDataInputStream implements FileCCacheConstants
{
    private static boolean DEBUG;
    
    public CCacheInputStream(InputStream is) {
        super(is);
    }
    
    public Tag readTag() throws IOException {
        char[] array = new char[1024];
        int read = -1;
        Object o = null;
        Object o2 = null;
        int i = this.read(2);
        if (i < 0) {
            throw new IOException("stop.");
        }
        if (i > array.length) {
            throw new IOException("Invalid tag length.");
        }
        while (i > 0) {
            read = this.read(2);
            int read2 = this.read(2);
            switch (read) {
                case 1: {
                    o = new Integer(this.read(4));
                    o2 = new Integer(this.read(4));
                    break;
                }
            }
            i -= 4 + read2;
        }
        return new Tag(i, read, o, o2);
    }
    
    public PrincipalName readPrincipal(int n) throws IOException, RealmException {
        int read;
        if (n == 1281) {
            read = 0;
        }
        else {
            read = this.read(4);
        }
        int length4 = this.readLength4();
        ArrayList o = new ArrayList();
        if (n == 1281) {
            --length4;
        }
        for (int i = 0; i <= length4; ++i) {
            o.add(new String(IOUtils.readExactlyNBytes(this, this.readLength4())));
        }
        if (o.isEmpty()) {
            throw new IOException("No realm or principal");
        }
        if (this.isRealm((String)o.get(0))) {
            String name = (String)o.remove(0);
            if (o.isEmpty()) {
                throw new IOException("No principal name components");
            }
            return new PrincipalName(read, (String[])o.toArray(new String[o.size()]), new Realm(name));
        }
        else {
            try {
                return new PrincipalName(read, (String[])o.toArray(new String[o.size()]), Realm.getDefault());
            }
            catch (RealmException ex) {
                return null;
            }
        }
    }
    
    boolean isRealm(String s) {
        try {
            Realm realm = new Realm(s);
        }
        catch (Exception ex) {
            return false;
        }
        StringTokenizer stringTokenizer = new StringTokenizer(s, ".");
        while (stringTokenizer.hasMoreTokens()) {
            String nextToken = stringTokenizer.nextToken();
            int i = 0;
            while (i < nextToken.length()) {
                if (nextToken.charAt(i) >= '\u008d') {
                    return false;
                }
                ++i;
            }
        }
        return true;
    }
    
    EncryptionKey readKey(int value) throws IOException {
        int read = this.read(2);
        if (value == 1283) {
            this.read(2);
        }
        return new EncryptionKey(IOUtils.readExactlyNBytes(this, this.readLength4()), read, new Integer(value));
    }
    
    long[] readTimes() throws IOException {
        return new long[] { (long)this.read(4) * 1000L, (long)this.read(4) * 1000L, (long)this.read(4) * 1000L, (long)this.read(4) * 1000L };
    }
    
    boolean readskey() throws IOException {
        return this.read() != 0;
    }
    
    HostAddress[] readAddr() throws IOException, KrbApErrException {
        int length4 = this.readLength4();
        if (length4 > 0) {
            ArrayList<HostAddress> list = new ArrayList();
            int i = 0;
            while (i < length4) {
                int read = this.read(2);
                int length5 = this.readLength4();
                if (length5 != 4 && length5 != 16) {
                    if (CCacheInputStream.DEBUG) {
                        System.out.println("Incorrect address format.");
                    }
                    return null;
                }
                else {
                    byte[] new_address = new byte[length5];
                    for (int j = 0; j < length5; ++j) {
                        new_address[j] = (byte)this.read(1);
                    }
                    list.add(new HostAddress(read, new_address));
                    ++i;
                }
            }
            return (HostAddress[])list.toArray(new HostAddress[list.size()]);
        }
        return null;
    }
    
    AuthorizationDataEntry[] readAuth() throws IOException {
        int length4 = this.readLength4();
        if (length4 > 0) {
            ArrayList<AuthorizationDataEntry> list = new ArrayList();
            for (int i = 0; i < length4; ++i) {
                list.add(new AuthorizationDataEntry(this.read(2), IOUtils.readExactlyNBytes(this, this.readLength4())));
            }
            return (AuthorizationDataEntry[])list.toArray(new AuthorizationDataEntry[list.size()]);
        }
        return null;
    }
    
    byte[] readData() throws IOException {
        int length4 = this.readLength4();
        if (length4 == 0) {
            return null;
        }
        return IOUtils.readExactlyNBytes(this, length4);
    }
    
    boolean[] readFlags() throws IOException {
        boolean[] array = new boolean[32];
        int read = this.read(4);
        if ((read & 0x40000000) == 0x40000000) {
            array[1] = true;
        }
        if ((read & 0x20000000) == 0x20000000) {
            array[2] = true;
        }
        if ((read & 0x10000000) == 0x10000000) {
            array[3] = true;
        }
        if ((read & 0x8000000) == 0x8000000) {
            array[4] = true;
        }
        if ((read & 0x4000000) == 0x4000000) {
            array[5] = true;
        }
        if ((read & 0x2000000) == 0x2000000) {
            array[6] = true;
        }
        if ((read & 0x1000000) == 0x1000000) {
            array[7] = true;
        }
        if ((read & 0x800000) == 0x800000) {
            array[8] = true;
        }
        if ((read & 0x400000) == 0x400000) {
            array[9] = true;
        }
        if ((read & 0x200000) == 0x200000) {
            array[10] = true;
        }
        if ((read & 0x100000) == 0x100000) {
            array[11] = true;
        }
        if (CCacheInputStream.DEBUG) {
            String s = ">>> CCacheInputStream: readFlags() ";
            if (array[1]) {
                s += " FORWARDABLE;";
            }
            if (array[2]) {
                s += " FORWARDED;";
            }
            if (array[3]) {
                s += " PROXIABLE;";
            }
            if (array[4]) {
                s += " PROXY;";
            }
            if (array[5]) {
                s += " MAY_POSTDATE;";
            }
            if (array[6]) {
                s += " POSTDATED;";
            }
            if (array[7]) {
                s += " INVALID;";
            }
            if (array[8]) {
                s += " RENEWABLE;";
            }
            if (array[9]) {
                s += " INITIAL;";
            }
            if (array[10]) {
                s += " PRE_AUTH;";
            }
            if (array[11]) {
                s += " HW_AUTH;";
            }
            System.out.println(s);
            return array;
        }
        else {
            return array;
        }
    }
    
    Object readCred(int n) throws IOException, RealmException, KrbApErrException, Asn1Exception {
        PrincipalName principal = null;
        try {
            principal = this.readPrincipal(n);
        }
        catch (Exception ex) {}
        if (CCacheInputStream.DEBUG) {
            System.out.println(">>>DEBUG <CCacheInputStream>  client principal is " + principal);
        }
        PrincipalName principal2 = null;
        try {
            principal2 = this.readPrincipal(n);
        }
        catch (Exception ex2) {}
        if (CCacheInputStream.DEBUG) {
            System.out.println(">>>DEBUG <CCacheInputStream> server principal is " + principal2);
        }
        EncryptionKey key = this.readKey(n);
        if (CCacheInputStream.DEBUG) {
            System.out.println(">>>DEBUG <CCacheInputStream> key type: " + key.getEType());
        }
        long[] times = this.readTimes();
        KerberosTime o = new KerberosTime(times[0]);
        Object o2 = (times[1] == 0L) ? null : new KerberosTime(times[1]);
        KerberosTime o3 = new KerberosTime(times[2]);
        Object o4 = (times[3] == 0L) ? null : new KerberosTime(times[3]);
        if (CCacheInputStream.DEBUG) {
            System.out.println(">>>DEBUG <CCacheInputStream> auth time: " + o.toDate().toString());
            System.out.println(">>>DEBUG <CCacheInputStream> start time: " + ((o2 == null) ? "null" : o2.toDate().toString()));
            System.out.println(">>>DEBUG <CCacheInputStream> end time: " + o3.toDate().toString());
            System.out.println(">>>DEBUG <CCacheInputStream> renew_till time: " + ((o4 == null) ? "null" : o4.toDate().toString()));
        }
        int readskey = this.readskey();
        TicketFlags o5 = new TicketFlags(this.readFlags());
        HostAddress[] addr = this.readAddr();
        Object o6 = null;
        if (addr != null) {
            o6 = new HostAddresses(addr);
        }
        AuthorizationDataEntry[] auth = this.readAuth();
        Object o7 = null;
        if (auth != null) {
            o7 = new AuthorizationData(auth);
        }
        byte[] data = this.readData();
        byte[] data2 = this.readData();
        if (principal == null || principal2 == null) {
            return null;
        }
        try {
            if (principal2.getRealmString().equals("X-CACHECONF:")) {
                String[] nameStrings = principal2.getNameStrings();
                if (nameStrings[0].equals("krb5_ccache_conf_data")) {
                    return new CredentialsCache.ConfigEntry(nameStrings[1], (nameStrings.length > 2) ? new PrincipalName(nameStrings[2]) : null, data);
                }
            }
            return new Credentials(principal, principal2, key, o, o2, o3, o4, (boolean)readskey, o5, o6, o7, (data != null) ? new Ticket(data) : null, (data2 != null) ? new Ticket(data2) : null);
        }
        catch (Exception ex3) {
            if (CCacheInputStream.DEBUG) {
                ex3.printStackTrace(System.out);
            }
            return null;
        }
    }
    
    static {
        CCacheInputStream.DEBUG = Krb5.DEBUG;
    }
}
